﻿<#
# AddRemoveUser.ps1
#
# A program that demonstrates PowerShell integration with UEM REST API.
#
# This sample program demonstrates how to use REST authorization, add a local user, remove user, and update local users login password..
# UEM REST api reference.  https:// help.blackberry.com/en/blackberry-web-services-for-blackberry-uem/current/
#>

$global:AuthorizationString = $null;
$global:UEMHostPortTenantGUIDBaseURL = $null;

<#
# <summary>
# Prompts users for information in order to generate authroization string to be used for authenticated REST route.
# </summary>
# <returns>Returns true when the setup is successful, and false otherwise.</returns>
#>
function Setup()
{
    $methodName = "Setup()";
    enterMethod($methodName);
    $provider = $null;

    <# Prompt the user for the UEM hostname, port, tentantGuid #>
    $UEMHostAndPort = Read-Host 'Enter UEM HostName:Port (E.g. UEM.host.name:18084) ';
    $TenantGuid = Read-Host 'Enter Tenant Id';

    $global:UEMHostPortTenantGUIDBaseURL = "https://"+$UEMHostAndPort + "/" + $TenantGuid+"/api/v1";

    do
    {
     try
        {
            [int]$authType = Read-Host "Authentication type: `n1) Active directory `n2) Local Account`n";
            if(($authType -eq 1) -or ($authType -eq 2))
            {
                $validNumber = $true;
            }
        }
        catch
        {
            $validNumber = $false;
        }
    }
    until
        ($validNumber)

    if($authType -eq 1)
    {
        $UserName = Read-Host 'Enter AD Username ';
        $Password = Read-Host 'Enter AD Password ';
        $Domain = Read-Host 'Enter Domain';
        $Provider = "AD";
    }
    elseif($authType -eq 2)
    {
        $UserName = Read-Host 'Enter local Username ';
        $Password = Read-Host 'Enter local Password ';
        $Provider = "LOCAL";
    }

    <# Convert Password to base64 #>
    $Bytes = [System.Text.Encoding]::UTF8.GetBytes($Password);
    $Base64Password = [Convert]::ToBase64String($Bytes);

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Content-Type", "application/vnd.blackberry.authorizationrequest-v1+json")

    <# Create body for REST call #>
    $body = @{}
    $body.add("provider",$Provider);
    $body.add("username", $UserName);
    $body.add("password", $Base64Password);
    if($authType -eq 1)
    {
        $body.add("domain", $Domain);
    }

    $request = $global:UEMHostPortTenantGUIDBaseURL + "/util/authorization";

    <# Invoke RESTMethod to call authorization route to generate the authorization headers #>
    try
    {
        $response = Invoke-RESTMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
        handleError $_;
    }

    $global:AuthorizationString = $response

    Write-Host([string]::Format("Authorization header string generated: {0}", $global:AuthorizationString));
    Write-Host("Setup Complete...");
    exitMethod($methodName);
    return $true;
}

<#
# <summary>
# Search for UEM user using the email address as the search parameter.
# </summary>
# <param>User email address</param>
# <returns>Returns response object based on the search parameter</returns>
#>
function getUEMUser($userEmailAddress)
 {

    $methodName = "getUEMUser()";
    enterMethod($methodName);
    $response = $null;

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.users-v1+json");

    $request = $global:UEMHostPortTenantGUIDURL+"/users?query=emailAddress="+$userEmailAddress;

    <# Invoke RESTMethod to call users API to obtain users in the system #>
    try
    {
        $response = Invoke-RESTMethod -Method GET -Headers $headers -Uri $request;
    }
    catch
    {
         handleError $_;
    }
    exitMethod($methodName);
    return $response;
 }

 <#
# <summary>
# Add local user to UEM Server
# </summary>
# <param>username, display name, first name, last name, email address, password obtained from the user input</param>
#>
function addLocalUser($userName, $displayName, $firstName, $lastName, $emailAddress, $password)
{
    $methodName = "addLocalUser()";
    enterMethod($methodName);

    $bytes = [System.Text.Encoding]::UTF8.GetBytes($password);
    $base64Password = [Convert]::ToBase64String($Bytes);

    <# Create and set the headers for the REST command #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.user-v1+json");

    <# Create body for REST call #>
    $body = @{};
    $body.Add("username",$userName);
    $body.add("displayName", $displayName);
    $body.add("firstName", $firstName);
    $body.add("lastName", $lastName);
    $body.add("emailAddress", $emailAddress);
    $body.Add("password", $base64Password);

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users";

    <# Invoke RESTMethod to create a local user. #>
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("User {0} was created successfully", $response.displayName));
    exitMethod($methodName);
}

<#
# <summary>
# Remove directory linked user from UEM Server.
# </summary>
# <param>User object obtained from get users call.</param>
#>
function removeDirectoryUser($user)
{
    $methodName = "removeDirectoryUser()";
    enterMethod($methodName);

    <# Create and set the headers for the REST command #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.users-v1+json");

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users/"+$user.users.guid;

    <# Invoke RESTMethod to remove UEM user. #>
    try
    {
        $response = Invoke-RestMethod -Method DELETE -Headers $headers -Uri $request;
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("User {0} was removed successfully", $user.users.emailAddress));
    exitMethod($methodName);
}

<#
# <summary>
# Convert password into base64.
# </summary>
# <param>password in plain text</param>
# <returns>Base64 of the password</returns>
#>
function base64Password($password)
{
	enterMethod($methodName);
    $bytes = [System.Text.Encoding]::UTF8.GetBytes($password);
    $base64Password = [Convert]::ToBase64String($Bytes);
    exitMethod($methodName);
    return $base64Password;
}

<#
# <summary>
# Update password for local UEM user..
# </summary>
# <param>User object from get users call.  Password from user input</param>
#>
function upatePassword($user, $password)
{
    $methodName = "upatePassword()";
    enterMethod($methodName);

    $base64Password = base64Password($password);
    <# Create and set the headers for the REST command #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.user-v1+json");

    $body = @{};
    $body.Add("password", $base64Password);

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users/"+$user.users.guid;

    <# Invoke RESTMethod to update local UEM users password #>
    try
    {
        $response = Invoke-RestMethod -Method PATCH -Headers $headers -Uri $request  -Body ($body | ConvertTo-Json);
    }
    catch
    {
         handleError $_;
    }

    Write-Host([string]::Format("Password was updated for {0}", $user.users.emailAddress));
    exitMethod($methodName);
}

<#
# <summary>
# Writes Entering method name
# </summary>
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
# <summary>
# Writes Exiting method name.
# </summary>
#<param>Method name</param>
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
# <summary>
# Handle any errors that occur when performing the REST call.  On error the program will exit.
# </summary>
# <param>The error</param>
# <return></return>
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}

<#
# <summary>
# The Main Function.
# </summary>
#>

function main
{
    $methodName = "main()";
    enterMethod($methodName);
    try
    {
        if (Setup)
        {
            do
            {
                try
                {
                [int]$actionType = Read-Host "1) Add local user to UEM `n2) Remove user from UEM`n3) Change local user login password`n";
                 if($actionType -eq 1 -or $actionType -eq 2 -or $actionType -eq 3)
                    {
                        $validNumber = $true;
                    }
                }
                catch
                {
                $validNumber = $false;
                }
            }
            until
                ($validNumber)

                if($actionType -eq 1)
                    {

                    do
                    {
                    $userName = Read-Host("Enter username");
                    $displayName = Read-Host("Enter display name");
                    $firstName = Read-Host("Enter first name");
                    $lastName = Read-Host("Enter last name");
                    $emailAddress = Read-Host("Enter email address");
                    $password = Read-Host("Enter login password") -AsSecureString;

                    if( [string]::IsNullOrWhiteSpace($userName) -or [string]::IsNullOrWhiteSpace($displayName) -or [string]::IsNullOrWhiteSpace($firstName) -or [string]::IsNullOrWhiteSpace($lastName) -or [string]::IsNullOrWhiteSpace($emailAddress) -or [string]::IsNullOrWhiteSpace($password))
                        {
                            Write-host("One or more values are empty.  All values must be filled in");
                            $validInput = $false;
                        }
                        else
                        {
                            $validInput = $true;
                        }
                    }
                    until ($validInput)
                    addLocalUser $userName $displayName $firstName $lastName $emailAddress $password
                }
                elseif($actionType -eq 2)
                {
                    do
                    {
                        $userEmailAddress = Read-Host "Enter email address of the directory user to remove from UEM";
                        if([string]::IsNullOrWhiteSpace($userEmailAddress))
                        {
                            $validInput = $false;
                        }
                        else
                        {
                            $validInput = $true;
                        }
                    }
                    until($validInput)

                    $getUserResponse = getUEMUser($userEmailAddress);
                    if($getUserResponse.users.length -eq 1)
                    {
                        Write-Host([string]::Format("User with email address {0} was found", $getUserResponse.users.emailAddress));
                        <# call removeDirectoryUser method to removed user from UEM #>
                        removeDirectoryUser($getUserResponse);
                    }
                    elseif($getUserResponse.users.length -gt 1)
                    {
                        Write-Host([string]::Format("More then one user was found with email address {0}", $userEmailAddress));
                    }
                    elseif($getUserResponse.users.length -eq 0)
                    {
                        Write-Host([string]::Format("No user was found with email address {0}", $userEmailAddress));
                    }
                }
                elseif($actionType -eq 3)
                {
                    do
                    {
                    $userEmailAddress = Read-Host "Enter email address of local user";
                    $newPassword = Read-Host("Enter new login password");

                        if([string]::IsNullOrWhiteSpace($userEmailAddress) -or [string]::IsNullOrEmpty($newPassword))
                        {
                            $validInput = $false;
                        }
                        else
                        {
                            $validInput = $true;
                        }
                    }
                    until($validInput)

                    $getUserResponse = getUEMUser($userEmailAddress);
                    if($getUserResponse.users.length -eq 1)
                    {
                        Write-Host([string]::Format("User with email address {0} was found", $getUserResponse.users.emailAddress));
                        <# Directory-linked users use their directory-configured password,#>
                        if([string]::IsNullOrWhiteSpace($getUserResponse.users.directoryId))
                        {
                            <# call removeDirectoryUser method to add User to UEM #>
                            upatePassword $getUserResponse $newPassword;
                        }
                        else
                        {
                            Write-Host([string]::Format("{0} is a directory user. Only local users are supported with this call", $userEmailAddress));
                        }
                    }
                    elseif($getUserResponse.users.length -gt 1)
                    {
                        Write-Host([string]::Format("More then one user was found with email address {0}", $userEmailAddress));
                    }
                    elseif($getUserResponse.users.length -eq 0)
                    {
                        Write-Host([string]::Format("No user was found with email address {0}", $userEmailAddress));
                    }
                }
        }
    }
    catch
    {
        handleError $_;
    }
    exitMethod($methodName);
}

<#
# <summary>
# Calls main function.
# </summary>
#>
main;