<#
# SetActivaitonPassword.ps1
#
# A program that demonstrates PowerShell integration with UEM REST API.
#
# This sample program demonstrates how to use REST api to serach for UEM user and set, replace and expire activation passwords.
# UEM REST api reference.  https:// help.blackberry.com/en/blackberry-web-services-for-blackberry-uem/current/
#>

$global:AuthorizationString = $null;
$global:UEMHostPortTenantGUIDBaseURL = $null;

<#
# <summary>
# Prompts users for information in order to generate authroization string to be used for authenticated REST route.
# </summary>
# <returns>Returns true when the setup is successful, and false otherwise.</returns>
#>
function Setup()
{
    $methodName = "Setup()";
    enterMethod($methodName);
    $provider = $null;

    <# Prompt the user for the UEM hostname, port, tentantGuid #>
    $UEMHostAndPort = Read-Host 'Enter UEM HostName:Port (E.g. UEM.host.name:18084) ';
    $TenantGuid = Read-Host 'Enter Tenant Id';

    $global:UEMHostPortTenantGUIDBaseURL = "https://"+$UEMHostAndPort + "/" + $TenantGuid+"/api/v1";

    do
    {
     try
        {
            [int]$authType = Read-Host "Authentication type: `n1) Active directory `n2) Local Account`n";

            if(($authType -eq 1) -or ($authType -eq 2))
            {
                $validNumber = $true;
            }
        }
        catch
        {
            $validNumber = $false;
        }
    }
    until
        ($validNumber)

    if($authType -eq 1)
    {
        $UserName = Read-Host 'Enter AD Username ';
        $Password = Read-Host 'Enter AD Password ';
        $Domain = Read-Host 'Enter Domain';
        $provider = "AD";
    }
    elseif($authType -eq 2)
    {
        $UserName = Read-Host 'Enter local Username ';
        $Password = Read-Host 'Enter local Password ';
        $provider = "LOCAL";
    }


    <# Convert Password to base64 #>
    $Bytes = [System.Text.Encoding]::UTF8.GetBytes($Password);
    $Base64Password = [Convert]::ToBase64String($Bytes);

    <# Create and set the headers for the REST request #>
    $headers = @{}
    $headers.Add("Content-Type", "application/vnd.blackberry.authorizationrequest-v1+json")

    <# Create body for REST call #>
    $body = @{}
    $body.add("provider",$provider);
    $body.add("username", $UserName);
    $body.add("password", $Base64Password);
    if($authType -eq 1)
    {
        $body.add("domain", $Domain);
    }

    $request = $global:UEMHostPortTenantGUIDUR+"/util/authorization";

    <# Invoke RESTMethod to call authorization route to generate the authorization headers #>
    try
    {
        $response = Invoke-RESTMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
        Write-Host($request);
    }
    catch
    {
        handleError $_;
    }

    $global:AuthorizationString = $response

    Write-Host([string]::Format("Authorization header string generated: {0}", $global:AuthorizationString));
    Write-Host("Setup Complete...");
    exitMethod($methodName);
    return $true;
}

<#
# <summary>
# Search for UEM user using the email address as the search parameter.
# </summary>
# <param>User email address</param>
# <returns>Returns the response object based on the search parameter</returns>
#>
 function getUser($userEmailAddress)
 {

    $methodName = "serachForDirectoryUser()";
    enterMethod($methodName);

    <# Create and set the headers for the REST command #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.users-v1+json");

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users?query=emailAddress="+$userEmailAddress;

     <# Invoke RESTMethod to Search for users in all configured company directories. #>
    try
    {
        $response = Invoke-RestMethod -Method GET -Headers $headers -Uri $request;
    }
    catch
    {
         handleError $_;
    }

    exitMethod($methodName);
    return($response);
 }

<#
# <summary>
# Set's an auto generated password for the user with a specific expiry and will also expire after use.
# Activation email message will be sent if STMP server is configured within UEM.
# </summary>
# <param>User GUID obatained from get users call</param>
# <returns>Return true if the api call was a success</returns>
#>
function setActivationPassword($userGuid)
{
    $methodName = "setActivationPassword()";
    enterMethod($methodName);

    <# Create and set the headers for the REST command #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.activationpasswords-v1+json");

    <# Create body for REST call #>
    $body = @{};
    $innerBody = @(@{
    "password" = $null;
    "emailTemplate" = $null;
    "expiry" = "2017-10-01T8:20:30.45-05:00";
    "expireAfterUse" = "true";
    "activationProfile"= $null;
    });
    $body.Add("activationPasswords",$innerBody);

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users/"+$userGuid+"/activationPasswords";

     <# Invoke RESTMethod to set activation password. #>
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
         handleError $_;
    }

    exitMethod($methodName);
    return $true;
}


<#
# <summary>
# Expire all activation passwords for a user. An expired activation password is no longer valid to be used for device activation
# </summary>
# <param>User GUID obtained from get users call</param>
# <returns>Return true if the api call was a success</returns>
#>
function expireActivationPasswords($userGuid)
{
    $methodName = "expireActivationPasswords()";
    enterMethod($methodName);

    <# Create and set the headers for the REST command #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.activationpasswords-v1+json");

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users/"+$userGuid+"/activationPasswords";

     <# Invoke RESTMethod to exipre all activation passwords. #>
    try
    {
        $response = Invoke-RestMethod -Method DELETE -Headers $headers -Uri $request
    }
    catch
    {
         handleError $_;
    }

    exitMethod($methodName);
    return $true;
}

<#
# <summary>
# Replace all activation password for a user. Existing passwords will be expired.
# Activation email message will be sent if STMP server is configured within UEM.
# </summary>
# <param>User GUID obtained from get users call</param>
# <returns>Returns true if command was successful</returns>
#>
function replaceActivationPassword($userGuid)
{
    $methodName = "replaceActivationPassword()";
    enterMethod($methodName);

    <# Create and set the headers for the REST command #>
    $headers = @{}
    $headers.Add("Authorization", $global:AuthorizationString);
    $headers.Add("Content-Type", "application/vnd.blackberry.activationpasswords-v1+json");

    <# Create body for REST call #>
    $body = @{};
    $innerBody = @(@{
    "password" = $null;
    "emailTemplate" = $null;
    "expiry" = "2017-09-01T19:20:30.45-05:00";
    "expireAfterUse" = "true";
    "activationProfile"= $null;
    });
    $body.Add("activationPasswords",$innerBody);

    $request = $global:UEMHostPortTenantGUIDBaseURL+"/users/"+$userGuid+"/activationPasswords";

     <# Invoke RESTMethod to set activation password. #>
    try
    {
        $response = Invoke-RestMethod -Method PUT -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
         handleError $_;
    }

    exitMethod($methodName);
    return $true;
}

<#
# <summary>
# Writes Entering method name
# </summary>
# <param>Method name</param>
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
# <summary>
# Writes Exiting method name.
# </summary>
# <param>Method name</param>
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
# <summary>
# Handle any errors that occur when performing the REST call.  On error the program will exit.
# </summary>
# <param>The error</param>
# <return></return>
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}


<#
# <summary>
# The Main Function.
# </summary>
#>
function main
{
    $methodName = "Main()";
    enterMethod($methodName);
    try
    {
        if (Setup)
        {
            $userEmailAddress = Read-Host "Enter email address of user";
            $getUserResponse = getUser($userEmailAddress);

            if($getUserResponse.users.length -eq 1)
            {
                Write-Host([string]::Format("User with email address {0} was found", $getUserResponse.users.emailAddress));
                $userGUID = $getUserResponse.users.guid;
                Write-Host("Choose activaiton password action:");
            do
            {
                try
                {
                [int]$actionType = Read-Host " 1) Set one random generated password`n 2) Replace and expire all activation passwords`n 3) Expire all activation passwords`n";
                 if($actionType -eq 1 -or $actionType -eq 2 -or $actionType -eq 3)
                    {
                        $validNumber = $true;
                    }
                }
                catch
                {
                $validNumber = $false;
                }
            }
                until
                ($validNumber)

                if($actionType -eq 1)
                {
                    if(setActivationPassword($userGUID))
                    {
                       Write-Host([string]::Format("Random activation password has been set for user {0}", $userEmailAddress));
                    }
                }
                elseif($actionType -eq 2)
                {
                    if(replaceActivationPassword($userGUID))
                    {
                        Write-Host([string]::Format("Activation password has been replaced for user {0}. Existing passwords have been expired.", $userEmailAddress ));
                    }
                }
                elseif($actionType -eq 3)
                {
                    if(expireActivationPasswords($userGUID))
                    {
                      Write-Host([string]::Format("All activation passwords expired for user {0}", $userEmailAddress));
                    }
                }
            }
            elseif($getUserResponse.users.length -gt 1)
            {
                Write-Host("More then one user was found");
            }
            elseif($getUserResponse.users.length -eq 0)
            {
                Write-Host("No user was found with eamil address");
            }
        }
    }
    catch
    {
        handleError $_;
    }
    exitMethod($methodName);
}

<#
# <summary>
# Calls main function.
# </summary>
#>
main;